import {
  expect,
  sourcesPath,
  fetchMock,
  mockConfigurations,
  Config,
} from './support/setup.js';

const ObjectResource = require(`${sourcesPath}/object_resource`).default;

describe('ObjectResource', () => {

  class Test extends ObjectResource {

    static get RESOURCE_NAME() {
      return 'test';
    }

    static get BASE_URL() {
      return '/tests';
    }

  }

  beforeEach(() => {
    mockConfigurations();
  });

  afterEach(() => {
    fetchMock.reset()
    fetchMock.restore()
  });

  describe('.create', () => {
    it('send the correct method and request', () => {
      fetchMock.postOnce('https://api.liveqa.io/api/v1/tests', { body: [{ id: 42 }], headers: {} });

      Test.create({ name: 'test' }).then((response) => {
        expect(response).to.have.deep.members([{ id: 42 }]);
      });
    });

    it('with error from the server', () => {
      fetchMock.postOnce('https://api.liveqa.io/api/v1/tests', { body: [{ id: 42 }], status: 422, headers: {} });

      Test.create({ name: 'test' }).catch((response) => {
        expect(response.message).to.eq('Unprocessable Entity');
      });
    });

    it('raise if no api token are given', () => {
      Config.configurations.accountToken = null;

      expect(() => { Test.create({ name: 'test' }) }).to.throw('Your API access are not set. Please check the documentation.');
    });
  });

  describe('.update', () => {
    it('send the correct method and request', () => {
      fetchMock.patchOnce('https://api.liveqa.io/api/v1/tests/42', { body: [{ id: 42 }], headers: {} });

      Test.update(42, { name: 'test' }).then((response) => {
        expect(response).to.have.deep.members([{ id: 42 }]);
      });
    });

    it('with error from the server', () => {
      fetchMock.patchOnce('https://api.liveqa.io/api/v1/tests/42', { body: [{ id: 42 }], status: 422, headers: {} });

      Test.update(42, { name: 'test' }).catch((response) => {
        expect(response.message).to.eq('Unprocessable Entity');
      });
    });
  });

  describe('.delete', () => {
    it('send the correct method and request', () => {
      fetchMock.deleteOnce('https://api.liveqa.io/api/v1/tests/42', { body: [{ id: 42 }], headers: {} });

      Test.delete(42, { name: 'test' }).then((response) => {
        expect(response).to.have.deep.members([{ id: 42 }]);
      });
    });

    it('with error from the server', () => {
      fetchMock.deleteOnce('https://api.liveqa.io/api/v1/tests/42', { body: [{ id: 42 }], status: 422, headers: {} });

      Test.delete(42, { name: 'test' }).catch((response) => {
        expect(response.message).to.eq('Unprocessable Entity');
      });
    });
  });

  describe('._headers', () => {
    it('valid headers', () => {
      const expectedHeaders = {
        'Accept': 'application/json',
        'Access-Control-Allow-Origin': '*',
        'Content-Type': 'application/json',
        'X-Account-Token': 'acc_xx',
        'X-Environment-Name': 'test',
        'X-Space-Name': 'LiveQA',
      };

      expect(Test._headers).to.deep.equal(expectedHeaders);
    });
  });

});
